#!/bin/bash
#
# Intel VCA Software Stack (VCASS)
#
# Copyright(c) 2015 Intel Corporation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
# General Public License for more details.
#
# The full GNU General Public License is included in this distribution in
# the file called "COPYING".
#
# Intel VCA Scripts.
#
unset LANG


### GLOBALS ###

export VCA_DRACUT_SRC_DIR=dracut_module
export VCA_DRACUT_DST_DIR=/lib/dracut/modules.d
export VCA_DRACUT_MNT_LOOP_DEV="50mountloopdev"
export VCA_DRACUT_NETWORK="40network"
export VCA_DRACUT_NFS="95nfs"
export VCA_CHECKSUMSFILE=sha1_checksums.txt


### PRE-CONFIGURATION FUNCTIONS ###

show_help () {

	echo "Usage: $0 [OPTIONS]"
	echo "Options:
--image-type <type>	Selects image creation strategy compatible with:
				volatile-bm	- volatile baremetal
				persistent-bm	- persistent baremetal
				volatile-dom0	- Xen dom0
				domu		- Xen domU
--kernel <version>	Kernel version name, used to generate a valid grub config
--version <version>	Version appended to image name
--grub-cfg <file>	Grub config file to be embedded in OS image
--ks-file <file>	Kickstarter file used to create OS image
--compressor <name>	Optional external compression engine to be passed to dracut as --compress
--add-rootfs		Optional dump rootfs img (as tag.gz)
--image-name <name>	Name of the output image
--out-dir <directory>	Directory where generated images are stored
--tmp-dir <directory>	Helper directory to store files while creating image by image_creator
--os-repo <path>	Repository containing rpms from CentOS image
--extras-repo <path>	Repository containing extra rpms, out of build (like additional, 3rd party validation tools)
--build-repo <path>	Repository containing rpms from the build
"
}

parse_parameters () {
	if [ "$1" == "-h" ] || [ "$1" == "--help" ] ; then
        show_help
		exit 0
	fi
	
	while [ "$1" != "" ]
	do
		case "$1" in
			--image-type)
				export VCA_IMG_TYPE="$2"
				shift
				shift
				;;
			--kernel)
				export VCA_KERNEL_NAME="$2"
				shift
				shift
				;;
			--version)
				export VCA_VERSION_NAME="$2"
				shift
				shift
				;;
			--grub-cfg)
				export VCA_GRUB_CFG="$2"
				shift
				shift
				;;
			--ks-file)
				export VCA_KS_FILE="$2"
				shift
				shift
				;;
			--compressor)
				export VCA_DRACUT_COMPRESSOR="$2"
				shift
				shift
				;;
			--add-rootfs)
				export VCA_ADD_ROOTFS="1"
				shift
				;;
			--image-name)
				export VCA_ISO_IMG="$2"
				shift
				shift
				;;
			--out-dir)
				export VCA_OUT_DIR="$2"
				shift
				shift
				;;
			--tmp-dir)
				export VCA_TMP_DIR="$2"
				shift
				shift
				;;
			--os-repo)
				export VCA_OS_REPO="$2"
				shift
				shift
				;;
			--extras-repo)
				export VCA_EXTRAS_REPO="$2"
				shift
				shift
				;;
			--build-repo)
				export VCA_BUILD_REPO="$2"
				shift
				shift
				;;
            -h)
                show_help
                exit 0
                ;;
            --help)
                show_help
                exit 0
                ;;
			*)
				echo "unknown parameter '$1'"
				exit 1
				;;
		esac
	done
}

setup_parameters () {
	if [ -z "$VCA_IMG_TYPE" ]; then
		echo "Choosing exactly one --image-type by option or global variable is mandatory!"
		exit 1
	else
		check_image_type
	fi
	if [ -z "$VCA_VERSION_NAME" ]; then
		export VCA_VERSION_NAME="_0.0.0"
	else
		export VCA_VERSION_NAME="_$VCA_VERSION_NAME"
	fi
	if [ -z "$VCA_GRUB_CFG" ]; then
		export VCA_GRUB_CFG="grub_cfgs/grub_production_baremetal.cfg"
	fi
	if [ -z "$VCA_KS_FILE" ]; then
		export VCA_KS_FILE="ks_files/vv_production_baremetal.ks"
	fi
	if [ -z "$VCA_ISO_IMG" ]; then
		get_image_name_if_empty
	fi
	if [ -z "$VCA_OUT_DIR" ]; then
		export VCA_OUT_DIR="out"
	fi
	if [ -z "$VCA_TMP_DIR" ]; then
		export VCA_TMP_DIR="tmp"
	fi
	if [ -z "$VCA_ADD_ROOTFS" ]; then
		if [ "$VCA_IMG_TYPE" == "persistent-bm" ]; then
			export VCA_ADD_ROOTFS="1"
		else
			export VCA_ADD_ROOTFS="0"
		fi
	fi
	if [ -z "$VCA_OS_REPO" ]; then
		export VCA_OS_REPO="file:/usr/lib/vca/repos/CentOS7.1/os_repo"
	fi
	if [ -z "$VCA_EXTRAS_REPO" ]; then
		export VCA_EXTRAS_REPO="file:/usr/lib/vca/repos/CentOS7.1/extras_repo"
	fi
	if [ -z "$VCA_BUILD_REPO" ]; then
		export VCA_BUILD_REPO="file:/usr/lib/vca/repos/build_repo"
	fi
	if [ -z "$VCA_KERNEL_NAME" ]; then
		get_kernel_name_from_repo
	else
		check_if_kernel_name_exist
	fi
}

check_image_type () {
	if ! [ "$VCA_IMG_TYPE" == "volatile-bm" ] && ! [ "$VCA_IMG_TYPE" == "persistent-bm" ] && \
	   ! [ "$VCA_IMG_TYPE" == "volatile-dom0" ] && ! [ "$VCA_IMG_TYPE" == "domu" ]; then
		echo "Unknown --image-type! See help for details."
		exit 1
	fi
}

get_image_name_if_empty () {
	if [ "$VCA_IMG_TYPE" == "volatile-bm" ]; then
		export VCA_ISO_IMG="vca_baremetal"
	elif [ "$VCA_IMG_TYPE" == "persistent-bm" ]; then
		export VCA_ISO_IMG="vca_persistent"
	elif [ "$VCA_IMG_TYPE" == "volatile-dom0" ]; then
		export VCA_ISO_IMG="vca_xen"
	else
		export VCA_ISO_IMG="vca_domu"
	fi
}

get_kernel_name_from_repo () {
	VCA_REPO_PREFIX='file:'
	VCA_BUILD_REPO_DIR=`echo ${VCA_BUILD_REPO#$VCA_REPO_PREFIX}`

	export VCA_KERNEL_NAME=`find $VCA_BUILD_REPO_DIR -type f -name "kernel-3.10.0-[0-9]*.el7.[0-9]*.x86_64.rpm" | awk -F '-' '{print $2"-"$3}' | sed 's/.rpm//'`

	if [ "$VCA_KERNEL_NAME" == "" ]; then
	    export VCA_KERNEL_NAME=`find $VCA_BUILD_REPO_DIR -type f -name "kernel-[0-9]*.x86_64.rpm" | awk -F '-' '{print $2}' | tr '_' '-'`
	fi
}

check_if_kernel_name_exist () {
	VCA_KERNEL_NAME_TMP=$VCA_KERNEL_NAME
	get_kernel_name_from_repo
	if ! [ "$VCA_KERNEL_NAME" == "$VCA_KERNEL_NAME_TMP" ]; then
		echo "Provided kernel name ($VCA_KERNEL_NAME_TMP) not exist in $VCA_BUILD_REPO!"
	fi
}

summarize_parameters () {
	if [ "$VCA_IMG_TYPE" == "volatile-bm" ]; then
		VCA_FULL_IMG_NAME="Baremetal image"
	elif [ "$VCA_IMG_TYPE" == "persistent-bm" ]; then
		VCA_FULL_IMG_NAME="Persistent FS boot image"
	elif [ "$VCA_IMG_TYPE" == "volatile-dom0" ]; then
		VCA_FULL_IMG_NAME="Xen Dom0 image"
	else
		VCA_FULL_IMG_NAME="Xen DomU image"
	fi
	echo -e "\nParameters used to generate $VCA_FULL_IMG_NAME:"
	echo "VCA_IMG_TYPE=$VCA_IMG_TYPE"
	echo "VCA_KERNEL_NAME=$VCA_KERNEL_NAME"
	echo "VCA_VERSION_NAME=$VCA_VERSION_NAME"
	echo "VCA_GRUB_CFG=$VCA_GRUB_CFG"
	echo "VCA_KS_FILE=$VCA_KS_FILE"
	echo "VCA_DRACUT_COMPRESSOR=$VCA_DRACUT_COMPRESSOR"
	echo "VCA_ADD_ROOTFS=$VCA_ADD_ROOTFS"
	echo "VCA_ISO_IMG=$VCA_ISO_IMG"
	echo "VCA_OUT_DIR=$VCA_OUT_DIR"
	echo "VCA_TMP_DIR=$VCA_TMP_DIR"
	echo "VCA_OS_REPO=$VCA_OS_REPO"
	echo "VCA_EXTRAS_REPO=$VCA_EXTRAS_REPO"
	echo "VCA_BUILD_REPO=$VCA_BUILD_REPO"
}


### GENERATE IMAGE MAIN FUNCTION ###

export VCA_ROOTFS_NAME=vca_rootfs
export VCA_MNT_ISO_DIR=mounted_isoimg
export VCA_INITRAMFS_IMG=vca_initramfs.img 
export VCA_ROOTFS=$VCA_ROOTFS_NAME.img
export VCA_MNT_ROOTFS_DIR=mounted_rootfs
export VCA_CPIO_PATCH=cpio_patch.bin
export VCA_TMP_KS_FILE=vca_tmp_ks_file

generate_image () {
	generate_rootfs_image
	
	mount_rootfs_image
	
	generate_initramfs_image
	
	generate_boot_image
	
	prepare_mounted_image
	
	chroot_to_the_image
	
	copy_bootloader_file_to_image
	
	if [ "$VCA_ADD_ROOTFS" == "1" ]; then
		get_rootfs_as_tar_gz
	fi
	
	umount_images
	
	copy_output_to_out_dir
}


### GENERATE IMAGE HELPER FUNCTIONS ###

generate_rootfs_image () {
	echo -e "\nGenerate rootfs image\n"
	mkdir -p $VCA_TMP_DIR

	cp -f $VCA_KS_FILE $VCA_TMP_KS_FILE
	check_rc
	
	sed -i "s|VCA_OS_REPO|$VCA_OS_REPO|g" $VCA_TMP_KS_FILE
	sed -i "s|VCA_EXTRAS_REPO|$VCA_EXTRAS_REPO|g" $VCA_TMP_KS_FILE
	sed -i "s|VCA_BUILD_REPO|$VCA_BUILD_REPO|g" $VCA_TMP_KS_FILE
	
	image-creator -v $VCA_TMP_KS_FILE -n $VCA_ROOTFS_NAME -t $VCA_TMP_DIR
	check_rc
	rm -rf $VCA_TMP_DIR
	rm -rf $VCA_TMP_KS_FILE
}

mount_rootfs_image () {
	echo -e "\nMount the rootfs image, so kernel modules can be used in initramfs\n"
	umount $VCA_MNT_ROOTFS_DIR
	mkdir -p $VCA_MNT_ROOTFS_DIR
	mount $VCA_ROOTFS $VCA_MNT_ROOTFS_DIR
	check_rc
}

generate_initramfs_image () {
	echo -e "\nGenerate initramfs image\n"
	
	if ! [ -z "$VCA_DRACUT_COMPRESSOR" ]; then
		export VCA_DRACUT_COMPRESSOR_CMD="--compress $VCA_DRACUT_COMPRESSOR"
	fi
	
	if [ "$VCA_IMG_TYPE" == "volatile-bm" ] || [ "$VCA_IMG_TYPE" == "volatile-dom0" ]; then
		copy_custom_dracut_module $VCA_DRACUT_MNT_LOOP_DEV
		
		dracut -v --omit-drivers "plx87xx plx87xx_dma" --add-drivers "loop ext4 pl2303" --add mountloopdev --include \
		  `pwd`/$VCA_ROOTFS_NAME.img /root/root_partition.img `pwd`/$VCA_INITRAMFS_IMG  $VCA_KERNEL_NAME \
		  -f -k `pwd`/$VCA_MNT_ROOTFS_DIR/lib/modules/$VCA_KERNEL_NAME \
		  --early-microcode $VCA_DRACUT_COMPRESSOR_CMD --no-hostonly
		check_rc
		
		remove_custom_dracut_module $VCA_DRACUT_MNT_LOOP_DEV
		
	elif [ "$VCA_IMG_TYPE" == "persistent-bm" ]; then
		copy_custom_dracut_module $VCA_DRACUT_NETWORK
		copy_custom_dracut_module $VCA_DRACUT_NFS
		
		# vca.conf on host must be exchanged in order to build dracut image correctly
		VCA_MODPROBE_VCA_FILE=/etc/modprobe.d/vca.conf
		
		if [ -f "$VCA_MODPROBE_VCA_FILE" ]; then
			cp "$VCA_MODPROBE_VCA_FILE" ./vca.conf.bak
			cp persistent_files/vca.conf "$VCA_MODPROBE_VCA_FILE"
		fi
		
		dracut -v -f \
		  --add 'nfs' \
		  --add-drivers 'nfs nfsv3 nfsv4 virtio_net virtio_console loop ext4 plx87xx plx87xx_dma vop vop_bus vca_csa' \
		  --include `pwd`/persistent_files/blacklist /etc/modprobe.d/blacklist \
		  --include `pwd`/persistent_files/vca.conf /etc/modprobe.d/vca.conf \
		  --include `pwd`/persistent_files/91-netup.rules /etc/udev/rules.d/91-netup.rules \
		  --include `pwd`/persistent_files/vca_up.sh /sbin/vca_up.sh \
		  --install 'hostname awk cut grep xargs logger tail' \
		  --prefix `pwd`/$VCA_MNT_ROOTFS_DIR \
		  --kmoddir `pwd`/$VCA_MNT_ROOTFS_DIR/lib/modules/$VCA_KERNEL_NAME \
		  `pwd`/$VCA_INITRAMFS_IMG $VCA_KERNEL_NAME \
		  --early-microcode $VCA_DRACUT_COMPRESSOR_CMD \
          --no-hostonly
		check_rc
		
		if [ -f ./vca.conf.bak ]; then
			cp ./vca.conf.bak "$VCA_MODPROBE_VCA_FILE"
			rm ./vca.conf.bak
		fi
		remove_custom_dracut_module $VCA_DRACUT_NETWORK
		remove_custom_dracut_module $VCA_DRACUT_NFS
	elif [ "$VCA_IMG_TYPE" == "domu" ]; then
		sync
		dracut -v --add-drivers "xen-blkfront ext4" `pwd`/$VCA_INITRAMFS_IMG  $VCA_KERNEL_NAME \
		  -f -k `pwd`/$VCA_MNT_ROOTFS_DIR/lib/modules/$VCA_KERNEL_NAME \
		  --early-microcode $VCA_DRACUT_COMPRESSOR_CMD --no-hostonly
		check_rc
		sync
	fi
}

generate_boot_image () {
	echo "Create complete image container"
	if ! [ "$VCA_IMG_TYPE" == "domu" ]; then
		export VCA_IMAGE_SIZE=$((`ls $VCA_INITRAMFS_IMG -l --block-size=512 | awk '{print $5}'`))
		echo "Initramfs size: $VCA_IMAGE_SIZE * 512B blocks"
		export VCA_ISO_IMG_EXT=".bin"
	else
		export VCA_IMAGE_SIZE=$((`ls $VCA_ROOTFS -l --block-size=512 | awk '{print $5}'`))
		echo "RootFS size: $VCA_IMAGE_SIZE * 512B blocks"
		export VCA_ISO_IMG_EXT=".img"
	fi
	if [ "$VCA_IMG_TYPE" == "persistent-bm" ]; then
		export VCA_COUNT=$(($VCA_IMAGE_SIZE+2*1024*20))
	else
		export VCA_COUNT=$(($VCA_IMAGE_SIZE+2*1024*50))
	fi
	echo "Image size: $VCA_COUNT *512 B blocks"
	dd if=/dev/zero of=$VCA_ISO_IMG$VCA_ISO_IMG_EXT count=$VCA_COUNT
	check_rc
    sync; sync
}

prepare_mounted_image () {
	echo -e "\nCreate partition in the container\n"
	VCA_LOOPDEV=`losetup -f --show $VCA_ISO_IMG$VCA_ISO_IMG_EXT`
	
	# clean-up to make sure if dir is empty
	umount $VCA_MNT_ISO_DIR
	rm -rf $VCA_MNT_ISO_DIR
	mkdir -p $VCA_MNT_ISO_DIR
	
	if [ "$VCA_IMG_TYPE" == "domu" ]; then
		mkfifo command_fifo
		fdisk $VCA_ISO_IMG$VCA_ISO_IMG_EXT < command_fifo &
		sleep 3
		echo -e 'p\nn\np\n1\n\n\np\nw\n' >> command_fifo
		sleep 10
		for job in `jobs -p`
		do
			echo "fdisk hasn't finished for 10 seconds, exiting"
            ps
			#exit 1
		done
		sync
		rm command_fifo
		
		kpartx -s -v -a $VCA_LOOPDEV
		
		sync
		
		VCA_LOOPDEVPARTITION=/dev/mapper/`echo $VCA_LOOPDEV | cut -d'/' -f 3`p1
		
		VCA_COUNTER=10
		while [ ! -e "$VCA_LOOPDEVPARTITION" ] && [ $VCA_COUNTER -gt 0 ] ; do
			sleep 1
			VCA_COUNTER=$(expr $VCA_COUNTER - 1)
			echo "Sleeping $VCA_COUNTER"
		done
		
		if [ ! -e "$VCA_LOOPDEVPARTITION" ]; then
			echo "No $VCA_LOOPDEVPARTITION"
			exit 1
		fi
		
		mkfs.ext4 $VCA_LOOPDEVPARTITION
		check_rc
        sync; sync
		
		mount $VCA_LOOPDEVPARTITION $VCA_MNT_ISO_DIR
		
		echo "Copy RootFS to the image"
		cp -r $VCA_MNT_ROOTFS_DIR/* $VCA_MNT_ISO_DIR
		check_rc
		
		echo "Copy InitramFS to the image"
		cp $VCA_INITRAMFS_IMG $VCA_MNT_ISO_DIR/boot
		check_rc
	else
		mkfs.fat $VCA_LOOPDEV
		check_mount_umount_rc
        sync; sync
		
		mount $VCA_LOOPDEV $VCA_MNT_ISO_DIR
		check_mount_umount_rc
		
		cp $VCA_MNT_ROOTFS_DIR/boot/vmlinuz* $VCA_MNT_ISO_DIR
		check_mount_umount_rc
		
		if [ "$VCA_IMG_TYPE" == "volatile-dom0" ]; then
			cp $VCA_MNT_ROOTFS_DIR/boot/xen*.gz $VCA_MNT_ISO_DIR
			check_mount_umount_rc
		fi
		
		cp $VCA_INITRAMFS_IMG $VCA_MNT_ISO_DIR
		check_mount_umount_rc
	fi
}

chroot_to_the_image () {
	echo -e "\nChroot to the image\n"
	if ! [ "$VCA_IMG_TYPE" == "domu" ]; then
		umount $VCA_MNT_ISO_DIR
		VCA_MNT_ISO_DIR=$VCA_MNT_ROOTFS_DIR/boot
		mount $VCA_LOOPDEV $VCA_MNT_ISO_DIR
		check_mount_umount_rc
		
		mount -o bind /dev $VCA_MNT_ROOTFS_DIR/dev/
		check_mount_umount_rc
		
		mount -t proc proc $VCA_MNT_ROOTFS_DIR/proc/
		if [ $? -ne 0 ]; then
			umount $VCA_MNT_ROOTFS_DIR/dev/
			umount $VCA_MNT_ISO_DIR
			umount $VCA_MNT_ROOTFS_DIR
			losetup -v -d $VCA_LOOPDEV
			exit 1
		fi
		
		chroot  $VCA_MNT_ROOTFS_DIR /usr/bin/env VCA_LOOPDEV=$VCA_LOOPDEV /bin/bash -x <<'EOF'
		
		echo "Install bootloader in the image at $VCA_LOOPDEV"
		echo "set root=(hd0)" > /etc/grub.cfg
		echo "configfile (hd0)/grub.cfg" >> /etc/grub.cfg
		mkdir -p /boot/EFI/BOOT
		grub2-mkstandalone -d /usr/lib/grub/x86_64-efi/ -O x86_64-efi --fonts="unicode" -o /boot/EFI/BOOT/BOOTX64.EFI /boot/grub/grub.cfg=/etc/grub.cfg
EOF
		if [ $? -ne 0 ]; then
			umount $VCA_MNT_ROOTFS_DIR/dev/
			umount $VCA_MNT_ROOTFS_DIR/proc/
			umount $VCA_MNT_ISO_DIR
			umount $VCA_MNT_ROOTFS_DIR
			losetup -v -d $VCA_LOOPDEV
			exit 1
		fi
		
		echo "Leave chroot"
		umount $VCA_MNT_ROOTFS_DIR/dev/
		umount $VCA_MNT_ROOTFS_DIR/proc/
	else
		mount -o bind /dev $VCA_MNT_ISO_DIR/dev/
		check_rc
		mount -t proc proc $VCA_MNT_ISO_DIR/proc/
		check_rc
		chroot  $VCA_MNT_ISO_DIR /usr/bin/env VCA_LOOPDEV=$VCA_LOOPDEV /bin/bash -x <<'EOF'

		echo "Install bootloader in the image at $VCA_LOOPDEV"
		grub2-install $VCA_LOOPDEV
EOF
		check_rc
		
		echo "Leave chroot"
		umount $VCA_MNT_ISO_DIR/dev/
		umount $VCA_MNT_ISO_DIR/proc/
	fi
}

copy_bootloader_file_to_image () {
	echo -e "\nCopy bootloader config file to the image\n"
	if ! [ "$VCA_IMG_TYPE" == "domu" ]; then
		sed -e s/VCA_KERNEL_VERSION/"$VCA_KERNEL_NAME"/g $VCA_GRUB_CFG > $VCA_MNT_ISO_DIR/grub.cfg
		check_rc
	else
		sed -e s/VCA_KERNEL_VERSION/"$VCA_KERNEL_NAME"/g $VCA_GRUB_CFG > $VCA_MNT_ISO_DIR/boot/grub2/grub.cfg
		check_rc
	fi
}

get_rootfs_as_tar_gz () {
	echo -e "\nCompress Rootfs tree\n"
	mkdir -p $VCA_OUT_DIR
	tar -zcf "${VCA_ROOTFS_NAME}_$VCA_ISO_IMG$VCA_VERSION_NAME-tree.tar.gz" -C "$VCA_MNT_ROOTFS_DIR" .
	check_rc
	
	echo "Copy ${VCA_ROOTFS_NAME}_$VCA_ISO_IMG$VCA_VERSION_NAME.tar.gz to output dir"
	cp "${VCA_ROOTFS_NAME}_$VCA_ISO_IMG$VCA_VERSION_NAME-tree.tar.gz" $VCA_OUT_DIR
	check_rc
}

umount_images () {
	echo -e "\nUmount the image $VCA_MNT_ISO_DIR\n"
	umount $VCA_MNT_ISO_DIR
	check_mount_umount_rc
	
	echo "Umount the image $VCA_MNT_ROOTFS_DIR"
	umount $VCA_MNT_ROOTFS_DIR
	check_mount_umount_rc
	
	if [ "$VCA_IMG_TYPE" == "domu" ]; then
		kpartx -s -d $VCA_LOOPDEV
    fi

	losetup -v -d $VCA_LOOPDEV
}

copy_output_to_out_dir () {
	echo -e "\nCopy outputs to $VCA_OUT_DIR\n"
	mkdir -p $VCA_OUT_DIR
	if ! [ "$VCA_IMG_TYPE" == "domu" ]; then
		cp $VCA_ISO_IMG$VCA_ISO_IMG_EXT $VCA_OUT_DIR/$VCA_ISO_IMG$VCA_VERSION_NAME.img
		check_rc
	else
		mv $VCA_ISO_IMG.img $VCA_ISO_IMG$VCA_VERSION_NAME.img
		tar -zcf $VCA_OUT_DIR/$VCA_ISO_IMG$VCA_VERSION_NAME.tar.gz $VCA_ISO_IMG$VCA_VERSION_NAME.img
		check_rc
	fi
}

# copy_custom_dracut_module $1
# $1 - custom module name
copy_custom_dracut_module () {
	echo "Copy custom module ($1) to dracut directory"
	VCA_DRACUT_MODULE=$1
	VCA_DRACUT_MODULE_DST_DIR=$VCA_DRACUT_DST_DIR/$VCA_DRACUT_MODULE
	if [ -d $VCA_DRACUT_MODULE_DST_DIR ]; then
		echo "Dracut module $VCA_DRACUT_MODULE_DST_DIR already exists; making a backup"
		rm -rf ./$VCA_DRACUT_MODULE.bak
		cp -r $VCA_DRACUT_MODULE_DST_DIR ./$VCA_DRACUT_MODULE.bak
		check_rc
	fi
	cp -r $VCA_DRACUT_SRC_DIR/$VCA_DRACUT_MODULE $VCA_DRACUT_DST_DIR
	check_rc
}

# remove_custom_dracut_module $1
# $1 - custom module name
remove_custom_dracut_module () {
	echo "Remove custom dracut module: $1"
	VCA_DRACUT_MODULE=$1
	rm -rf $VCA_DRACUT_DST_DIR/$VCA_DRACUT_MODULE
	check_rc
	if [ -d ./$VCA_DRACUT_MODULE.bak ]; then
		echo "Restore original dracut module from a local backup ./$VCA_DRACUT_MODULE.bak"
		mv ./$VCA_DRACUT_MODULE.bak $VCA_DRACUT_DST_DIR/$VCA_DRACUT_MODULE
		check_rc
	fi
}

check_mount_umount_rc () {
	if [ $? -ne 0 ]; then
		umount $VCA_MNT_ISO_DIR
		umount $VCA_MNT_ROOTFS_DIR
		losetup -v -d $VCA_LOOPDEV
		exit 1
	fi
}

check_rc () {
	if [ $? -ne 0 ]; then
		exit 1
	fi
}


### MAIN SCRIPT FLOW ###

# parse user provided parameters (the highest priority)
parse_parameters "$@"

# prepare required parameters based on already defined values in system and provided by user
setup_parameters

# print out parameters used to generate image 
summarize_parameters

# main function used to generate image
generate_image
